*-----------------------------------------------------------
* Program    : ***** SPACE WAR *****
* Written by : Charles Kelly
* Date       : April, 2014 v0.8
*
* Description: Spacewar was possibly the first graphical computer game ever written.
*
* Original authors were: Steve Russell, Martin Graetz and Wayne Witaenem.
* It was first coded on the PDP-1 in 1962 by Stephen Russell, Peter Samson, Dan Edwards,
* and Martin Graetz, together with Alan Kotok, Steve Piner, and Robert A Saunders. 
* Spacewar! is in the public domain, but this credit paragraph must accompany all
* distributed versions of the program.
*
* This version is my interpretation of their game for EASy68K.
*-----------------------------------------------------------

*-----------------------------------------------------------
* Rotate the x,y point contained in \1,\2 by angle in \3
* Post: d6 is x', d7 is y'
Rotate	macro
        move.w  \1,d3            	; x
        move.w  \2,d4            	; y
        move.w  \3,d1           	; a
        bsr     cosD1               ; d2 = cos(a)
        move.w  d3,d6               ; x
        muls.w  d2,d6               ; x * cos(a)
        move.w  d4,d7               ; y
        muls.w  d2,d7               ; y * cos(a)
        bsr     sinD1               ; d2 = sin(a)
        muls.w  d2,d4               ; y * sin(a)
        muls.w  d2,d3               ; x * sin(a)
        sub.l   d4,d6               ; x' * $4000
        add.l   d3,d7               ; y' * $4000
        * sin(a) and cos(a) are multiplied by $4000, shift left 2 bits to get word.word fixed point
        asl.l   #2,d6               ; x'
        asl.l   #2,d7               ; y'
        endm


*-----------------------------------------------------------
*-----------------------------------------------------------
	ORG	$1000
START:				; first instruction of program
   
* Load audio files
    lea     engine1_wav,a1  load engine1.wav
    move    #engine1,d1     sound cue
    move    #71,d0
    trap    #15

    lea     engine2_wav,a1  load engine2.wav
    move    #engine2,d1     sound cue
    move    #71,d0
    trap    #15

    lea     explode_wav,a1  load explode.wav
    move    #explode,d1     sound cue
    move    #71,d0
    trap    #15

    lea     torpFire_wav,a1 load torpedoFire.wav
    move    #torpFire,d1    sound cue
    move    #71,d0
    trap    #15

    lea     torpHit_wav,a1  load torpedoHit.wav
    move    #torpHit,d1     sound cue
    move    #71,d0
    trap    #15

* Display instructions
    move.l  #MENU_COLOR,d1
    move.l  #MENU_FONT,d2
    move.b  #21,d0              ; set font
    trap    #15
    lea     INSTRUCTIONS,a1
    move.b  #14,d0              ; display string
    trap    #15
    move.l  #100,d1
    move.b  #23,d0              ; delay 5 seconds
    trap    #15

    move.b  #11,D0
    move.w  #$FF00,D1
    trap    #15                 ; clear screen
    
* Enable double buffering
    move.b  #92,d0
    move.b  #17,d1
    trap    #15

    * Main game loop
    repeat

        move.b  #11,D0
        move.w  #$FF00,D1
        trap    #15                 ; clear screen
        
        * Display scores
        move.l  #BLUE,d1
        move.l  #MENU_FONT,d2
        move.b  #21,d0              ; set font
        trap    #15

        move.b  #11,d0              ; set cursor position
        move.w  #$0201,D1           ; column 2, row 1
        trap    #15
        move.b  #3,d0               ; display number
        move.l  score1,D1
        trap    #15

        move.l  #YELLOW,d1
        move.l  #MENU_FONT,d2
        move.b  #21,d0              ; set font
        trap    #15

        move.b  #11,d0              ; set cursor position
        move.w  #$2601,D1           ; column 10, row 1
        trap    #15
        move.b  #3,d0               ; display number
        move.l  score2,D1
        trap    #15

        * Draw Ships
        * 2d rotation about angle a
        * d6 = x' = (x * cos(a) - y * sin(a))
        * d7 = y' = (y * cos(a) + x * sin(a))

        * Draw ship1
        move.l  #BLUE,d1
        move.b  #80,d0              ; set pen color
        trap    #15
        lea     SHIP1,a0            ; ship1 points
		Rotate	(a0)+,(a0)+,angle1
        add.l   ship1x,d6           ; translated x'
        add.l   ship1y,d7           ; translated y'
        move.l  d6,d1
        swap    d1                  ; d1.w is x'
        move.l  d7,d2
        swap    d2                  ; d2.w is y'
        move.b  #86,d0              ; move to X,Y
        trap    #15
        move.b  #85,d0              ; draw line to X,Y
        repeat
            Rotate	(a0)+,(a0)+,angle1
            add.l   ship1x,d6           ; translated x'
            add.l   ship1y,d7           ; translated y'
            move.l  d6,d1
            swap    d1                  ; d1.w is x'
            move.l  d7,d2
            swap    d2                  ; d2.w is y'
            trap    #15
        until A0 <HS> #SHIP1_END do.s
        
        * Draw torpedo1
        if.b torp1On <EQ> #TRUE then.l
            lea TORPEDO1,a0             ; torpedo points
    		Rotate	(a0)+,(a0)+,torp1Ang
            add.l   torp1x,d6           ; translated x'
            add.l   torp1y,d7           ; translated y'
            move.l  d6,d1
            swap    d1                  ; d1.w is x'
            move.l  d7,d2
            swap    d2                  ; d2.w is y'
            move.b  #86,d0              ; move to X,Y
            trap    #15
            move.b  #85,d0              ; draw line to X,Y
            repeat
    			Rotate	(a0)+,(a0)+,torp1Ang
                add.l   torp1x,d6           ; translated x'
                add.l   torp1y,d7           ; translated y'
                move.l  d6,d1
                swap    d1                  ; d1.w is x'
                move.l  d7,d2
                swap    d2                  ; d2.w is y'
                trap    #15
            until A0 <HS> #TORPEDO1_END do.s
        endi

        * Draw ship2
        move.l  #YELLOW,d1
        move.b  #80,d0              ; set pen color
        trap    #15
        lea SHIP2,a0                ; ship2 points
		Rotate	(a0)+,(a0)+,angle2
        add.l   ship2x,d6           ; translated x'
        add.l   ship2y,d7           ; translated y'
        move.l  d6,d1
        swap    d1                  ; d1.w is x'
        move.l  d7,d2
        swap    d2                  ; d2.w is y'
        move.b  #86,d0              ; move to X,Y
        trap    #15
        move.b  #85,d0              ; draw line to X,Y
        repeat
			Rotate	(a0)+,(a0)+,angle2
            add.l   ship2x,d6           ; translated x'
            add.l   ship2y,d7           ; translated y'
            move.l  d6,d1
            swap    d1                  ; d1.w is x'
            move.l  d7,d2
            swap    d2                  ; d2.w is y'
            trap    #15
        until A0 <HS> #SHIP2_END do.s
       
        * Draw torpedo2
        if.b torp2On <EQ> #TRUE then.l
            lea TORPEDO2,a0             ; torpedo points
            Rotate	(a0)+,(a0)+,torp2Ang
            add.l   torp2x,d6           ; translated x'
            add.l   torp2y,d7           ; translated y'
            move.l  d6,d1
            swap    d1                  ; d1.w is x'
            move.l  d7,d2
            swap    d2                  ; d2.w is y'
            move.b  #86,d0              ; move to X,Y
            trap    #15
            move.b  #85,d0              ; draw line to X,Y
            repeat
    			Rotate	(a0)+,(a0)+,torp2Ang
                add.l   torp2x,d6           ; translated x'
                add.l   torp2y,d7           ; translated y'
                move.l  d6,d1
                swap    d1                  ; d1.w is x'
                move.l  d7,d2
                swap    d2                  ; d2.w is y'
                trap    #15
            until A0 <HS> #TORPEDO2_END do.s
        endi
        
        * Get users input
        move.l  #'A'<<24+'S'<<16+'D'<<8+'F',D1  ; check for keypress (a,s,d,f)
        move.b  #19,D0
        trap    #15
        if.l d1 <ne> #0 then.l      ; if key press
            btst.l  #24,d1          ; test for 'a'
            if <ne> then.s          ; if 'a'
                move.w  angle1,d0
                sub.w   #1,d0       ; turn left
                if <mi> then.s
                    add.w   #360,d0
                endi
                move.w  d0,angle1
            endi
            btst.l  #16,d1          ; test for 's'
            if <ne> then.s          ; if 's'
                move.w  angle1,d0
                add.w   #1,d0       ; turn right
                if.w d0 <hs> #360 then.s
                    sub.w   #360,d0
                endi
                move.w  d0,angle1
            endi
            btst.l  #8,d1           ; test for 'd'
            if <ne> then.s          ; if 'd' Thrust
                * Thrust Vx = Vx + cos(a), Vy = Vy + sin(a)
                clr.l   d2
                move.l  d1,-(sp)    ; save d1
                move.w  angle1,d1
                bsr     cosD1
                asr     #4,d2       ; /16  small thrust
                ext.l	d2
                add.l   d2,ship1Vx
                bsr     sinD1
                asr     #4,d2       ; /16
                ext.l	d2
                add.l   d2,ship1Vy
                move    #engine1,d1 ; engine1 sound cue
                move    #72,d0      ; play engine1 sound
                trap    #15
                move.l  (sp)+,d1    ; restore d1
            endi
          
            * If fire torpedo1
            btst.l  #0,d1           ; test for 'f'
            if <ne> and.w torp1TTL <eq> #0 then.s 	; if 'f' Fire torpedo AND ready to fire
               	move.w  angle1,torp1Ang     ; set torpedo direction
	            move.l	ship1X,torp1X       ; set torpedo location
	            move.l	ship1Y,torp1Y
	            move.b	#TRUE,torp1On	    ; enable torpedo
                * Thrust Vx = Vx + cos(a), Vy = Vy + sin(a)
                clr.l   d2
                move.w  torp1Ang,d1
                bsr     cosD1
                ext.l	d2
                asl.l   #3,d2       ; *8 thrust
                move.l  d2,torp1Vx
                bsr     sinD1
                ext.l	d2
                asl.l   #3,d2       ; *8 thrust
                move.l  d2,torp1Vy
                move.w  #TORP_TTL,torp1TTL  ; set time to live
                move    #torpFire,d1    ; play torpedo fire sound
                move    #72,d0
                trap    #15
            endi
        endi
        
        move.l  #'K'<<24+'L'<<16+$BA<<8+$DE,D1  ; check for keypress (k,l,;,')
        move.b  #19,D0
        trap    #15
        if.l d1 <ne> #0 then.l      ; if key press
            btst.l  #24,d1          ; test for 'k'
            if <ne> then.s          ; if 'k'
                move.w  angle2,d0
                sub.w   #1,d0       ; turn left
                if <mi> then.s
                    add.w   #360,d0
                endi
                move.w  d0,angle2
            endi
            btst.l  #16,d1          ; test for 'l'
            if <ne> then.s          ; if 'l'
                move.w  angle2,d0
                add.w   #1,d0       ; turn right
                if.w d0 <hs> #360 then.s
                    sub.w   #360,d0
                endi
                move.w  d0,angle2
            endi
            btst.l  #8,d1           ; test for ';'
            if <ne> then.s          ; if ';' Thrust
                * Thrust Vx = Vx + cos(a), Vy = Vy + sin(a)
                clr.l   d2
                move.l  d1,-(sp)    ; save d1
                move.w  angle2,d1
                bsr     cosD1
                asr     #4,d2       ; /16  small thrust
                ext.l   d2
                add.l   d2,ship2Vx
                bsr     sinD1
                asr     #4,d2       ; /16
                ext.l   d2
                add.l   d2,ship2Vy
                move    #engine2,d1 ; play engine2 sound
                move    #72,d0
                trap    #15
                move.l  (sp)+,d1    ; restore d1
            endi
            
            * If fire torpedo2
            btst.l  #0,d1           ; test for (')
            if <ne> and.w torp2TTL <eq> #0 then.s 	; if (') Fire torpedo AND ready to fire
               	move.w  angle2,torp2Ang     ; set torpedo direction
	            move.l	ship2X,torp2X       ; set torpedo location
	            move.l	ship2Y,torp2Y
	            move.b	#TRUE,torp2On	    ; enable torpedo
                * Thrust Vx = Vx + cos(a), Vy = Vy + sin(a)
                clr.l   d2
                move.w  torp2Ang,d1
                bsr     cosD1
                ext.l	d2
                asl.l   #3,d2       ; *8 thrust
                move.l  d2,torp2Vx
                bsr     sinD1
                ext.l	d2
                asl.l   #3,d2       ; *8 thrust
                move.l  d2,torp2Vy
                move.w  #TORP_TTL,torp2TTL  ; set time to live
                move    #torpFire,d1    ; play torpedo fire sound
                move    #72,d0
                trap    #15
            endi
        endi
    
        * Update torpedo1 positions
        if.b torp1On <eq> #TRUE then.l
            move.l  torp1Vx,d0
            add.l   d0,torp1x
            if.w torp1x <lt> #-SHIP_WIDTH/2 then.s  ; if off screen left 
                move.w  #GAME_WIDTH+SHIP_WIDTH/2,torp1x
            else 
                if.w torp1x <gt> #GAME_WIDTH+SHIP_WIDTH/2 then.s  ; if off screen right
                    move.w  #-SHIP_WIDTH/2,torp1x
                endi
            endi
            move.l  torp1Vy,d0
            add.l   d0,torp1y
            if.w torp1y <lt> #-SHIP_WIDTH/2 then.s            ; if off screen top
                move.w  #GAME_HEIGHT+SHIP_WIDTH/2,torp1y
            else
                if.w torp1y <gt> #GAME_HEIGHT+SHIP_WIDTH/2 then.s ; if off screen bottom
                    move.w  #-SHIP_WIDTH/2,torp1y
                endi
            endi
            add.w   #1,torp1Ang                     ; rotate torpedo
            if.w torp1Ang <gt> #360 then.s
                sub.w   #360,torp1Ang
            endi
        endi
        
        * Update torpedo2 positions
        if.b torp2On <eq> #TRUE then.l
            move.l  torp2Vx,d0
            add.l   d0,torp2x
            if.w torp2x <lt> #-SHIP_WIDTH/2 then.s  ; if off screen left 
                move.w  #GAME_WIDTH+SHIP_WIDTH/2,torp2x
            else 
                if.w torp2x <gt> #GAME_WIDTH+SHIP_WIDTH/2 then.s  ; if off screen right
                    move.w  #-SHIP_WIDTH/2,torp2x
                endi
            endi
            move.l  torp2Vy,d0
            add.l   d0,torp2y
            if.w torp2y <lt> #-SHIP_WIDTH/2 then.s            ; if off screen top
                move.w  #GAME_HEIGHT+SHIP_WIDTH/2,torp2y
            else
                if.w torp2y <gt> #GAME_HEIGHT+SHIP_WIDTH/2 then.s ; if off screen bottom
                    move.w  #-SHIP_WIDTH/2,torp2y
                endi
            endi
            add.w   #1,torp2Ang                     ; rotate torpedo
            if.w torp2Ang <gt> #360 then.s
                sub.w   #360,torp2Ang
            endi
        endi
        
        * Decrement torpedo time to live
        if.w torp1TTL <hi> #0 then.s
            sub.w   #1,torp1TTL     ; decrement time to live
            if <eq> then.s          ; if torpedo time expired
                move.b  #FALSE,torp1On  ; turn off
            endi
        endi
        if.w torp2TTL <hi> #0 then.s
            sub.w   #1,torp2TTL
            if <eq> then.s          ; if torpedo time expired
                move.b  #FALSE,torp2On  ; turn off
            endi
        endi

        * Update ship1 positions
        move.l  ship1Vx,d0
        add.l   d0,ship1x
        if.w ship1x <lt> #-SHIP_WIDTH/2 then.s  ; if off screen left 
            move.w  #GAME_WIDTH+SHIP_WIDTH/2,ship1x
        else 
            if.w ship1x <gt> #GAME_WIDTH+SHIP_WIDTH/2 then.s  ; if off screen right
                move.w  #-SHIP_WIDTH/2,ship1x
            endi
        endi
        move.l  ship1Vy,d0
        add.l   d0,ship1y
        if.w ship1y <lt> #-SHIP_WIDTH/2 then.s            ; if off screen top
            move.w  #GAME_HEIGHT+SHIP_WIDTH/2,ship1y
        else
            if.w ship1y <gt> #GAME_HEIGHT+SHIP_WIDTH/2 then.s ; if off screen bottom
                move.w  #-SHIP_WIDTH/2,ship1y
            endi
        endi
        
        * Update ship2 positions
        move.l  ship2Vx,d0
        add.l   d0,ship2x
        if.w ship2x <lt> #-SHIP_WIDTH/2 then.s  ; if off screen left 
            move.w  #GAME_WIDTH+SHIP_WIDTH/2,ship2x
        else 
            if.w ship2x <gt> #GAME_WIDTH+SHIP_WIDTH/2 then.s  ; if off screen right
                move.w  #-SHIP_WIDTH/2,ship2x
            endi
        endi
        move.l  ship2Vy,d0
        add.l   d0,ship2y
        if.w ship2y <lt> #-SHIP_WIDTH/2 then.s            ; if off screen top
            move.w  #GAME_HEIGHT+SHIP_WIDTH/2,ship2y
        else
            if.w ship2y <gt> #GAME_HEIGHT+SHIP_WIDTH/2 then.s ; if off screen bottom
                move.w  #-SHIP_WIDTH/2,ship2y
            endi
        endi

        * Check for collisions
        * Perform circle collision
        * Pre: \1=address of X1.L, \2=address of Y1.L, \3=address of Radius1.L
        *      \4=address of X2.L, \5=address of Y1.L, \6=address of Radius1.L
        * Post: D1.L=0 if no collision, 1 if collision
collideCircle MACRO
        movem.l d0/a0,-(sp)         ; save working registers
        lea     \1,a0               ; address of X1
        move.l  (a0),d0             ; D0 = X1
        lea     \4,a0               ; address of X2
        sub.l   (a0),d0             ; D0 = X1 - X2
        clr.w   d0                  ; convert from fixed point nnnn.nnnn to int word
        swap    d0
        muls    d0,d0               ; D0 = X difference squared
        lea     \2,a0               ; address of Y1
        move.l  (a0),d1             ; D1 = X1
        lea     \5,a0               ; address of Y2
        sub.l   (a0),d1             ; D1 = Y1 - Y2
        clr.w   d1                  ; convert from fixed point nnnn.nnnn to int word
        swap    d1
        muls    d1,d1               ; D1 = Y difference squared
        add.l   d1,d0               ; D0 = X difference squared + Y difference squared
        lea     \3,a0               ; address of Radius1
        move.l  (a0),d1             ; D1 = Radius1
        lea     \6,a0               ; address of Radius2
        add.l   (a0),d1             ; D1 = Radius1 + Radius2
        clr.w   d1                  ; convert from fixed point nnnn.nnnn to int word
        swap    d1
        muls    d1,d1               ; D1 = sum of radii squared
        if.l d0 <le> d1 then.s      ; if circles are colliding
            move.l  #1,d1           ; collision = true
        else.s
            clr.l   d1              ; collision = false
        endi
        movem.l (sp)+,d0/a0         ; restore registers
        ENDM

        if.b torp2On <eq> #TRUE then.s
            collideCircle   ship1x,ship1y,ship1r,torp2x,torp2y,torp2r
            if d1 <ne> #0 then.s        ; if collision between ship1 and torpedo2
                move.b	#FALSE,torp2On
                add.l   #1,score2
                move    #torpHit,d1     ; play torpedo hit sound
                move    #72,d0
                trap    #15
            endi
        endi
        
        if.b torp1On <eq> #TRUE then.s
            collideCircle   ship2x,ship2y,ship2r,torp1x,torp1y,torp1r
            if d1 <ne> #0 then.s        ; if collision between ship2 and torpedo1
                move.b	#FALSE,torp1On
                add.l   #1,score1
                move    #torpHit,d1     ; play torpedo hit sound
                move    #72,d0
                trap    #15
            endi
        endi
        
        move.b  #94,d0  ; repaint screen
        trap    #15
        
        move.b    #23,d0
        move.l    #1,d1    ; delay 1/100 sec
        trap    #15
    
    until.b quit <eq> #TRUE do.l

    MOVE.B    #9,D0
    TRAP    #15        ; halt simulator

* sine D1
* Pre: D1.W is angle in degrees (0 to 360)
* Post: D2.W is sin(angle) * $4000
* sin 90 to 180 = sin(180 - a)
* sin 180 to 270 = -sin(a - 180)
* sin 270 to 360 = -sin(360 - a)
sinD1
    movem.l d0/d1/a0,-(sp)
    if.w d1 <hs> #360 then.s
        sub.w   #360,d1
    endi
    lea     SINE_TABLE,a0
    if.w d1 <hs> #0 and.w d1 <ls> #90 then.s    ; if angle is 0 - 90
        lsl.l   #1,d1           ; * 2 for word table
        move.w  (a0,d1.w),d2
    else.s
    if.w d1 <hi> #90 and.w d1 <ls> #180 then.s  ; angle 91 - 180
        move.w  #180,d0
        sub.w   d1,d0           ; 180 - angle
        lsl.l   #1,d0           ; * 2 for word table
        move.w  (a0,d0.w),d2
    else.s
    if.w d1 <hi> #180 and.w d1 <ls> #270 then.s ; angle 181 - 270
        sub.w   #180,d1         ; angle - 180
        lsl.l   #1,d1           ; * 2 for word table
        move.w  (a0,d1.w),d2
        neg.w   d2
    else.s                                      ; angle 271 - 360
        move.w  #360,d0
        sub.w   d1,d0           ; 360 - angle
        lsl.l   #1,d0           ; * 2 for word table
        move.w  (a0,d0.w),d2
        neg.w   d2
    endi
    endi
    endi
    movem.l (sp)+,d0/d1/a0
    rts
    
* cosine D1
* Pre: D1.W is angle in degrees (0 to 360)
* Post: D2.W is cos(angle) * $4000
* cos(a) = sin(a + 90)
cosD1
    move.l  d1,-(sp)
    add.w   #90,d1      ; (a + 90)
    bsr     sinD1
    move.l  (sp)+,d1
    rts


* Variables and Strings
engine1_wav  dc.b   'engine1.wav',0    sound file
engine2_wav  dc.b   'engine2.wav',0
explode_wav  dc.b   'explode.wav',0
torpFire_wav dc.b   'torpedoFire.wav',0
torpHit_wav  dc.b   'torpedoHit.wav',0
engine1On    dc.b   FALSE
engine2On    dc.b   FALSE

engine1  equ    0   sound cue
engine2  equ    1
explode  equ    2
torpFire equ    3
torpHit  equ    4

score1   dc.l   0           player 1 score
score2   dc.l   0           player 2 score

ship1x   dc.l   $00300000   fixed point word.word
ship1y   dc.l   $00A00000   fixed point
angle1   dc.w   90
ship1Vx  dc.l   0           fixed point
ship1Vy  dc.l   0           fixed point
ship1r   dc.l   $000F0000   collision radius
ship2x   dc.l   $02400000   fixed point
ship2y   dc.l   $00A00000   fixed point
angle2   dc.w   90
ship2Vx  dc.l   0           fixed point
ship2Vy  dc.l   0           fixed point
ship2r   dc.l   $000F0000   collision radius

TORP_TTL equ    400         torpedo time to live
torp1x   dc.l   $00200000   fixed point word.word
torp1y   dc.l   $00200000   fixed point
torp1Ang dc.w   0
torp1Vx  dc.l   0           fixed point
torp1Vy  dc.l   0           fixed point
torp1r   dc.l   $00050000   collision radius
torp1TTL dc.w   0           time to live
torp1On  dc.b   FALSE
         ds.w   0           ; force even address
torp2x   dc.l   $00200000   fixed point word.word
torp2y   dc.l   $00A00000   fixed point
torp2Ang dc.w   0
torp2Vx  dc.l   0           fixed point
torp2Vy  dc.l   0           fixed point
torp2r   dc.l   $00050000   collision radius
torp2TTL dc.w   0           time to live
torp2On  dc.b   FALSE

quit     dc.b   FALSE

GAME_WIDTH  equ 640
GAME_HEIGHT equ 480
TRUE    equ    1
FALSE   equ    0
CR      equ    $d
LF      equ    $a
GREEN   equ $00008000
YELLOW  equ $0000FFFF
BLUE    equ $00FF0000 
MENU_COLOR  equ $00008000
MENU_FONT   equ $04120001
INSTRUCTIONS    
        dc.b    CR,LF,CR,LF,CR,LF,CR,LF
        dc.b    '                 Spacewar',CR,LF,CR,LF
        dc.b    '  ==== Ship 1 ====      ==== Ship 2 ====',CR,LF
        dc.b    '   a - turn left         k - turn left',CR,LF
        dc.b    '   s - turn right        l - turn right',CR,LF
        dc.b    '   d - thrust            ; - thrust',CR,LF
        dc.b    '   f - fire              '' - fire',CR,LF,0

SHIP_WIDTH  equ 30
SHIP_HEIGHT equ 16

        ds.w    0       ; force even word alignment
* The ships and torpedo are drawn as a series of connected line segments. The
* X,Y vertices below represent the coordinates of the vertices. Drawing starts
* at the first vertex and continues, in order, to each following vertex.
SHIP1: ; X,Y verticies
*  -15  -5  5   15 (X)
*-8  ____    ___
*-3  \__ \__/   \
* 5   __| __     |
*16  /___/  \___/
*(Y)         X, Y   X, Y   X, Y etc
    dc.w   -15,-8, -5,-8, -3,-3, 3,-3, 5,-8, 13,-8, 15,-3, 15,2, 13,7, 5,7, 3,2
    dc.w    -3,2, -5,7, -15,7, -13,2, -8,2, -8,-3, -13,-3, -15,-8
SHIP1_END

SHIP2:
*  -15  -5  5   15
*-8  ___    
*-3  \  \______/\
* 5  |   ______  |  
*16  /__/      \/
    dc.w    -15,-8, -7,-8, -5,-3, 10,-3, 13,-8, 15,-3, 15,2, 13,7
    dc.w     10,2, -5,2, -7,7 ,-15,7, -13,2, -13,-3, -15,-8
SHIP2_END

TORPEDO1:
* -5 0 5
*-5      
* 0 _\/_
* 5  /\   
    dc.w     0,0, -5,0, -2,-4, 0,0, 2,-4, 5,0, 0,0, 2,4, -2,4, 0,0
TORPEDO1_END

TORPEDO2:
* -5 0 5
*-5      
* 0 _\/_
* 5  /\   
    dc.w     0,0, -5,0, -2,-4, 0,0, 2,-4, 5,0, 0,0, 2,4, -2,4, 0,0
TORPEDO2_END

SINE_TABLE:  ; 0 thru 90 degrees multiplied by $4000 (shift right 14 bits to divide by $4000)
    dc.w    0,285,571,857,1142,1427,1712,1996,2280,2563
    dc.w    2845,3126,3406,3685,3963,4240,4516,4790,5062,5334
    dc.w    5603,5871,6137,6401,6663,6924,7182,7438,7691,7943
    dc.w    8192,8438,8682,8923,9161,9397,9630,9860,10086,10310
    dc.w    10531,10748,10963,11173,11381,11585,11785,11982,12175,12365
    dc.w    12550,12732,12910,13084,13254,13420,13582,13740,13894,14043
    dc.w    14188,14329,14466,14598,14725,14848,14967,15081,15190,15295
    dc.w    15395,15491,15582,15668,15749,15825,15897,15964,16025,16082
    dc.w    16135,16182,16224,16261,16294,16321,16344,16361,16374,16381
    dc.w    16384
    
    END    START        ; last line of source
























*~Font name~Courier New~
*~Font size~10~
*~Tab type~1~
*~Tab size~4~
